import * as Cmd from '../cmd';
import { Component } from '../index';
export * from './hash';
export * from './memoize';
import { ActionReturn, ActionCmdReturn, StandardActionReturn, ActionsType, InitObjReturn } from '../types';
export declare type Dt<T extends string, D = null> = {
    tag: T;
    data: D;
} & {
    __tsTag: 'DateType';
};
/**
 * ADT Helper for TS
 * e.g.
 * ```ts
 * type Msg =
 * | Dt<'fetchBook', number>
 * | Dt<'updateBook', Book>
 *
 * let msg = dt('fetchBook', 1)
 * switch(msg.tag) {
 *   case 'fetchBook':
 *      //...
 *      break
 *   case 'updateBook':
 *      //...
 *      break
 *   default:
 *      never(msg) // incomplete check from TS
 *      break
 * }
 * ```
 */
export declare function dt<T extends string, D = null>(tag: T, data?: D): Dt<T, D>;
/** @public */
export declare const never: (f: never) => never;
/** @public */
export declare function mkInit<S, A>(state: S, cmd?: Cmd.CmdType<A>): ActionCmdReturn<S, A>;
export declare type Fn1<T1, R> = (a1: T1) => R;
/** @public */
export declare function compose<T1, T2, R>(fn1: Fn1<T1, T2>, fn2: Fn1<T2, R>): Fn1<T1, R>;
export declare function compose<T1, T2, T3, R>(fn1: Fn1<T1, T2>, fn2: Fn1<T2, T3>, fn3: Fn1<T3, R>): Fn1<T1, R>;
export declare function compose<T1, T2, T3, T4, R>(fn1: Fn1<T1, T2>, fn2: Fn1<T2, T3>, fn3: Fn1<T3, T4>, fn4: Fn1<T4, R>): Fn1<T1, R>;
export declare function compose<T1, T2, T3, T4, T5, R>(fn1: Fn1<T1, T2>, fn2: Fn1<T2, T3>, fn3: Fn1<T3, T4>, fn4: Fn1<T4, T5>, fn5: Fn1<T5, R>): Fn1<T1, R>;
export declare function compose<T1, T2, T3, T4, T5, T6, R>(fn1: Fn1<T1, T2>, fn2: Fn1<T2, T3>, fn3: Fn1<T3, T4>, fn4: Fn1<T4, T5>, fn5: Fn1<T5, T6>, fn6: Fn1<T6, R>): Fn1<T1, R>;
export declare function compose<T1, T2, T3, T4, T5, T6, T7, R>(fn1: Fn1<T1, T2>, fn2: Fn1<T2, T3>, fn3: Fn1<T3, T4>, fn4: Fn1<T4, T5>, fn5: Fn1<T5, T6>, fn6: Fn1<T6, T7>, fn7: Fn1<T7, R>): Fn1<T1, R>;
export declare function compose<T1, T2, T3, T4, T5, T6, T7, T8, R>(fn1: Fn1<T1, T2>, fn2: Fn1<T2, T3>, fn3: Fn1<T3, T4>, fn4: Fn1<T4, T5>, fn5: Fn1<T5, T6>, fn6: Fn1<T6, T7>, fn7: Fn1<T7, T8>, fn8: Fn1<T8, R>): Fn1<T1, R>;
export declare function compose<T1, T2, T3, T4, T5, T6, T7, T8, T9, R>(fn1: Fn1<T1, T2>, fn2: Fn1<T2, T3>, fn3: Fn1<T3, T4>, fn4: Fn1<T4, T5>, fn5: Fn1<T5, T6>, fn6: Fn1<T6, T7>, fn7: Fn1<T7, T8>, fn8: Fn1<T8, T9>, fn9: Fn1<T9, R>): Fn1<T1, R>;
/** @public */
export declare function defaults<T>(value: T | null | undefined, defaults: T): T;
export interface CombinedComps<T extends {
    [k: string]: [Component, InitObjReturn<any, any>];
}, A extends {
    [k: string]: any;
}> {
    /** Combined state object for child components */
    state: {
        [k in keyof T]: T[k][1]['state'];
    };
    /** Combined and mapped cmd object for child components */
    cmd: Cmd.Sub<A>[];
    /**
     * Splited mapped cmd object for child components, useful for router config, so you can call init cmd when each page routing.
     *
     */
    cmds: {
        [k in keyof T]: Cmd.Sub<A>[];
    };
    /** Combined action object for child components */
    actions: {
        [k in keyof T]: T[k][0]['actions'];
    };
    /** Combined view function for child components */
    views: {
        [k in keyof T]: T[k][0]['view'];
    };
    /**
     * helper function for render routes views, so you can do
     * `subComps.render('somePage', state, actions)` instead of
     * `SomePage.views(state.somePage, actions.somePage)` or
     * `<SomePage.views
     *    state={state.somePage}
     *    actions={actions.somePage}
     *  />`
     */
    render: <K extends Extract<keyof T, keyof S>, S>(k: K, state: S, actions: ActionsType<S, any>) => any;
}
export declare function combine<T extends {
    [k: string]: [Component, InitObjReturn<any, any>];
}, A extends {
    [k: string]: any;
}>(arg: T, _acts?: A): CombinedComps<T, A>;
export declare function withParents<S, A, PS, PA, A1, A2>(action: (a1: A1, a2: A2) => (s: S, a: A) => any, wrapper?: (action: (a1: A1, a2: A2) => StandardActionReturn<S, A>, parentState: PS, parentActions: PA, state: S, actions: A) => ActionReturn<S, A>): any;
export declare function withParents<S, A, PS, PA, A1, A2, A3>(action: (a1: A1, a2: A2, a3: A3) => (s: S, a: A) => any, wrapper?: (action: (a1: A1, a2: A2, a3: A3) => StandardActionReturn<S, A>, parentState: PS, parentActions: PA, state: S, actions: A) => ActionReturn<S, A>): any;
export declare function withParents<S, A, PS, PA, A1, A2, A3, A4>(action: (a1: A1, a2: A2, a3: A3, a4: A4) => (s: S, a: A) => any, wrapper?: (action: (a1: A1, a2: A2, a3: A3, a4: A4) => StandardActionReturn<S, A>, parentState: PS, parentActions: PA, state: S, actions: A) => ActionReturn<S, A>): any;
export declare function withParents<S, A, PS, PA, A1, A2, A3, A4, A5>(action: (a1: A1, a2: A2, a3: A3, a4: A4, a5: A5) => (s: S, a: A) => any, wrapper?: (action: (a1: A1, a2: A2, a3: A3, a4: A4, a5: A5) => StandardActionReturn<S, A>, parentState: PS, parentActions: PA, state: S, actions: A) => ActionReturn<S, A>): any;
export declare function overrideAction<PS, PA, S, A, A1>(parentActions: PA, getter: (_: PA) => (a1: A1) => any, wrapper?: (a1: A1) => (action: <S = any, A = any>(a1: A1) => StandardActionReturn<S, A>, parentState: PS, parentActions: PA, state: S, actions: A) => ActionReturn<S, A>): any;
export declare function overrideAction<S, A, PS, PA, A1, A2>(parentActions: PA, getter: (_: PA) => (a1: A1, a2: A2) => any, wrapper?: (a1: A1, a2: A2) => (action: <S = any, A = any>(a1: A1, a2: A2) => StandardActionReturn<S, A>, parentState: PS, parentActions: PA, state: S, actions: A) => ActionReturn<S, A>): any;
export declare function overrideAction<S, A, PS, PA, A1, A2, A3>(parentActions: PA, getter: (_: PA) => (a1: A1, a2: A2, a3: A3) => any, wrapper?: (a1: A1, a2: A2, a3: A3) => (action: <S = any, A = any>(a1: A1, a2: A2, a3: A3) => StandardActionReturn<S, A>, parentState: PS, parentActions: PA, state: S, actions: A) => ActionReturn<S, A>): any;
export declare function overrideAction<S, A, PS, PA, A1, A2, A3, A4>(parentActions: PA, getter: (_: PA) => (a1: A1, a2: A2, a3: A3, a4: A4) => any, wrapper?: (a1: A1, a2: A2, a3: A3, a4: A4) => (action: <S = any, A = any>(a1: A1, a2: A2, a3: A3, a4: A4) => StandardActionReturn<S, A>, parentState: PS, parentActions: PA, state: S, actions: A) => ActionReturn<S, A>): any;
