import { hashAny } from './hash';
import * as LRUCache from 'lru-cache';
var cache = new LRUCache({
    max: 100,
    maxAge: 10 * 60 * 1000
});
export function memoizeBind(callback) {
    var args = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        args[_i - 1] = arguments[_i];
    }
    var hashStr = args.concat(callback).map(hashAny).join('|');
    var cached = cache.get(hashStr);
    if (!cached) {
        cached = function () {
            var args2 = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args2[_i] = arguments[_i];
            }
            return callback.apply(void 0, args.concat(args2));
        };
        cache.set(hashStr, cached);
    }
    return cached;
}
/**
 * memoize stateless component (function component) via only one cache, it works like pure component.
 */
export function memoizeOne(fn, excludes) {
    if (excludes === void 0) { excludes = ['children']; }
    var cache = null;
    var hash = null;
    return function (props) {
        var newHash = Object.keys(props)
            .sort()
            .filter(function (k) { return excludes.indexOf(k) < 0; })
            .map(function (k) { return hashAny(props[k]); })
            .join('|');
        if (newHash !== hash) {
            var r = fn(props);
            cache = r;
            hash = newHash;
        }
        return cache;
    };
}
export function setCacheOptions(opts) {
    cache = new LRUCache(opts);
}
//# sourceMappingURL=memoize.js.map