"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var cmd_1 = require("./cmd");
var utils_1 = require("./utils");
function inject(s, a) {
    var ctx = dispatcher.getContext();
    return ctx;
}
exports.inject = inject;
var dispatcher;
(function (dispatcher) {
    dispatcher.getResult = function () {
        var ctx = contextStack[contextStack.length - 1];
        if (!ctx)
            return;
        return {
            state: ctx._internals.newState,
            cmd: ctx._internals.cmd,
        };
    };
    dispatcher.getContext = function () {
        checkActive();
        return contextStack[contextStack.length - 1];
    };
    var checkActive = function () {
        if (!contextStack.length) {
            throw new RangeError("inject is called out of scope, please ensure it is called in a synchronized action.");
        }
    };
    var CmdHelper = /** @class */ (function () {
        function CmdHelper(setState, _mapper) {
            this.setState = setState;
            this._mapper = _mapper;
            this.none = cmd_1.Cmd.none;
        }
        CmdHelper.prototype.map = function (mapper) {
            return new CmdHelper(this.setState, mapper);
        };
        CmdHelper.prototype.addSub = function () {
            var subs = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                subs[_i] = arguments[_i];
            }
            return this.addBatch(subs);
        };
        CmdHelper.prototype.addBatch = function (subs) {
            this._addCmd(cmd_1.Cmd.ofSub, subs);
            return this;
        };
        CmdHelper.prototype.addFn = function (task, args, succeedAction, failedAction) {
            this._addCmd(cmd_1.Cmd.ofFn, arguments);
            return this;
        };
        CmdHelper.prototype.addPromise = function (task, args, succeedAction, failedAction) {
            this._addCmd(cmd_1.Cmd.ofPromise, arguments);
            return this;
        };
        CmdHelper.prototype._addCmd = function (fn, args) {
            var _this = this;
            var _a;
            var ctx = dispatcher.getContext();
            var cmd = fn.apply(void 0, args);
            if (this._mapper) {
                cmd = cmd.map(function (c) { return function (a) { return c(_this._mapper(a)); }; });
            }
            (_a = ctx._internals.cmd).push.apply(_a, cmd);
            return cmd;
        };
        return CmdHelper;
    }());
    dispatcher.CmdHelper = CmdHelper;
    var contextStack = [];
    var InjectContext = /** @class */ (function (_super) {
        tslib_1.__extends(InjectContext, _super);
        function InjectContext(props, setState) {
            var _this = _super.call(this, setState) || this;
            utils_1.set(_this, props);
            _this._internals = {
                newState: props.state,
                cmd: [],
            };
            _this.setState = setState;
            _this.Cmd = new CmdHelper(setState);
            return _this;
        }
        return InjectContext;
    }(CmdHelper));
    dispatcher.InjectContext = InjectContext;
    function makeContext(props) {
        var setState = function (state, callback) {
            ctx._internals.newState = state;
            props.setState(state, callback);
            return ctx;
        };
        var ctx = new InjectContext(props, setState);
        return ctx;
    }
    function active(props) {
        contextStack.push(makeContext(props));
    }
    dispatcher.active = active;
    function deactive() {
        contextStack.pop();
    }
    dispatcher.deactive = deactive;
})(dispatcher = exports.dispatcher || (exports.dispatcher = {}));
//# sourceMappingURL=dispatcher.js.map